import { ICommand } from "./command";
import { Dict } from "../util";
import { ReactionCollector, Message } from "discord.js";
import yaml from "yaml";
import { getRepository } from "typeorm";
import { Contest } from "@db/entity/Contest";
import { isNumber } from "util";
import emoji_regex from "emoji-regex";
import { client } from "../client";


const CHANNEL_ID_PATTERN = /<#(\d+)>/;

let pendingReactors : Dict<ReactionCollector> = {};

async function init() {

}

interface ContestCreationOptions {
    in?: string;
    duration?: string;
    announce_winners?: boolean;
    vote_reaction?: string;
}

const CONTEST_DEFAULTS : ContestCreationOptions = {
    duration: "1d",
    announce_winners: false,
    vote_reaction: "❤️"
};

const DURATION_MULTIPLIERS : Dict<number> = {
    "s" : 1000,
    "min": 60 * 1000,
    "h": 60 * 60 * 1000,
    "d": 24 * 60 * 60 * 1000,
    "mon": 30 * 24 * 60 * 60 * 1000,
    "y": 365 * 24 * 60 * 60 * 1000
};

const DURATION_REGEX_STR = `(\\d+) ?(${Object.keys(DURATION_MULTIPLIERS).join("|")})`;
const DURATION_REGEX = new RegExp(DURATION_REGEX_STR, "i");

function parseDuration(duration: string) : number | undefined {
    let match = DURATION_REGEX.exec(duration);

    if(match.length == 0)
        return undefined;

    let num = match[1];
    let unit = match[2];

    return +num * DURATION_MULTIPLIERS[unit];
}

async function createContest(msg: Message, info: ContestCreationOptions) {
    let dur = parseDuration(info.duration);
    if(!dur) {
        await msg.channel.send(`${msg.author.toString()} Duration format is invalid!`);
        return;
    }

    if(!msg.guild.emojis.find(e => e.toString() == info.vote_reaction) && !emoji_regex().exec(info.vote_reaction)) {
        await msg.channel.send(`${msg.author.toString()} The vote emote must be accessible by everyone on the server!`);
        return;
    }

    let repo = getRepository(Contest);

    let contest = await repo.findOne({
        where: { channel: info.in }
    });

    if(contest) {
        await msg.channel.send(`${msg.author.toString()} The channel already has a contest running!`);
        return;
    }

    contest = repo.create({
        channel: info.in,
        startDate: new Date(),
        endDate: new Date(Date.now() + dur),
        announceWinners: info.announce_winners,
        voteReaction: info.vote_reaction
    });

    await repo.save(contest);
    await msg.channel.send(`${msg.author.toString()} Started contest (ID: ${contest.id})`);
}

export default <ICommand>{
    commands: [
        {
            pattern: "create contest",
            action: async (m, contents) => {
                let message = m.content.trim().substr(client.user.toString().length).trim().substr("create contest".length).trim();
                // let message = contents.substr("create contest".length);
                let contestData : ContestCreationOptions = { ...CONTEST_DEFAULTS, ...yaml.parse(message) };

                if(!contestData.in)
                    contestData.in = m.channel.id;
                else {
                    let matches = CHANNEL_ID_PATTERN.exec(contestData.in);
                    if(matches.length == 0) {
                        await m.channel.send(`${m.author.toString()} I can't see such a channel!`);
                        return;
                    }
                    
                    let channelId = matches[1];
                    if(!m.guild.channels.exists("id", channelId)) {
                        await m.channel.send(`${m.author.toString()} This channel is not in the current guild!`);
                        return;
                    }

                    contestData.in = channelId;
                }

                await createContest(m, contestData);
            }
        },
        {
            pattern: "contests",
            action: async (m) => {
                await m.channel.send("Heck");
            }
        },
        {
            pattern: /end contest( (\d*))?/,
            action: async (m, contents, matches) => {
                await m.channel.send("Heck");
            }
        },
        {
            pattern: "announce winners",
            action: async (m, contents, matches) => {
                await m.channel.send("Heck");
            }
        }
    ],
    onStart: init,
};